/*
 * Image plugin to VDR (C++)
 *
 * (C) 2004 Andreas Brachold    <vdr04-at-deltab.de>
 *     2003 Kai Tobias Burwieck <kai@burwieck.net>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <signal.h>
#include <wait.h>

#include <linux/dvb/video.h>

#include "player-image.h"
#include "control-image.h"
#include "setup-image.h"
#include "data-image.h"
#include "errorimage.h"
#include "image.h"

#include <vdr/device.h>
#include <vdr/status.h>
#include <vdr/tools.h>


const char *g_szConvertScript = "convert.sh";
const char *g_szZoomScript    = "convert_zoom.sh";
const char *g_szJumpScript    = "convert_jump.sh";


//----------cImagePlayer-------------




cImagePlayer::cImagePlayer(cSlideShow *pCurSlideShow)
: cPlayer(
#if VDRVERSNUM >= 10308
 (ImageSetup.m_bLiveAudio != 0)?pmVideoOnly:
#endif
 pmAudioVideo
 )
, m_bActive(true)
, m_pCurSlideShow(pCurSlideShow)
{
  m_nCurrentImage = 0;
  m_nTotalImages = 0;
  m_bConvertRunning = false;
  if(m_pCurSlideShow)
  {
    m_nTotalImages = m_pCurSlideShow->Count();
    // get real Index of FirstImage
    unsigned int i = 0;
    bool bFound = false;
    if(m_pCurSlideShow->FirstImage()) {
      cImage* p = m_pCurSlideShow->cList < cImage >::First();
      for (;p && i < m_nTotalImages; ++i)
      {
        // find out first image
        if(0 == strcmp(::basename(p->Name()),m_pCurSlideShow->FirstImage()))
        {
          m_pCurImage = p;
          bFound = true;
          break;
        }
        p = m_pCurSlideShow->cList < cImage >::Next(p);
      }
    }
    if(bFound) {
      m_nCurrentImage = i+1;
    }
    else {
      m_pCurImage = m_pCurSlideShow->cList < cImage >::First();
      m_nCurrentImage = 1;
    }
  }
  Start();
}

bool cImagePlayer::GetIndex(int &nCurrent, int &nTotal, bool /*bSnapToIFrame*/)
{
  // Notify other status-plugins with one picture as one second
  nCurrent = m_nCurrentImage; 
  // Notify other status-plugins count of picture as totalcount of seconds
  nTotal = m_nTotalImages;

  return true;
}

cImagePlayer::~cImagePlayer()
{
  m_bActive = false;
  m_condVar.Broadcast();
  Cancel(2);

  Detach();
  // Remove Slideshow
  if(m_pCurSlideShow) {
    delete m_pCurSlideShow;
    m_pCurSlideShow = 0;
  }
}


void cImagePlayer::Activate(bool bOn)
{
  if(bOn)
  {
    if(m_pCurSlideShow && m_pCurImage)
        Convert(m_pCurImage->Name(), "");
  }
  else if(m_bActive) {
    m_bActive = false;
  }
}


bool cImagePlayer::NextImage(int nOffset)
{
  if(m_pCurImage && m_nCurrentImage < m_nTotalImages) {
    cImage *newcurr;
    for (int i = 0; (i < nOffset) && (m_nCurrentImage < m_nTotalImages); ++i) {
      newcurr = m_pCurSlideShow->cList < cImage >::Next(m_pCurImage);
      m_pCurImage = newcurr;
      ++m_nCurrentImage;
    }
    return m_pCurImage != NULL;
  } 
  else if(ImageSetup.AutoRepeat) {
    cImage *newcurr;
    
    newcurr = m_pCurSlideShow->cList < cImage >::First();
    m_pCurImage = newcurr;
    m_nCurrentImage = 1;
    
    return m_pCurImage != NULL;
  }
  return false;
}



bool cImagePlayer::PrevImage(int nOffset)
{
  if(m_pCurImage && m_nCurrentImage > 1) {
    cImage *newcurr;

    for (int i = 0; (i < nOffset) && (m_nCurrentImage > 1); ++i) {
        newcurr = m_pCurSlideShow->cList < cImage >::Prev(m_pCurImage);
        m_pCurImage = newcurr;
        --m_nCurrentImage;
    }
    return m_pCurImage != NULL;
  }
  else if(ImageSetup.AutoRepeat) {
    cImage *newcurr;

    newcurr = m_pCurSlideShow->cList < cImage >::Last();
    m_pCurImage = newcurr;
    m_nCurrentImage = m_nTotalImages;
      return m_pCurImage != NULL;
  }
  return false;
}


bool cImagePlayer::GotoImage(unsigned int nNewPictureIndex)
{
  if(m_pCurImage && nNewPictureIndex > 0 && nNewPictureIndex < m_nTotalImages) {
    cImage *newcurr;
    newcurr = m_pCurSlideShow->cList < cImage >::First();
    m_pCurImage = newcurr;
    m_nCurrentImage = 0;
    for (unsigned int i = 0; i < nNewPictureIndex; ++i) {
      newcurr = m_pCurSlideShow->cList < cImage >::Next(m_pCurImage);
      m_pCurImage = newcurr;
      ++m_nCurrentImage;
    }
    return m_pCurImage != NULL;
	}
  return false;
}

void cImagePlayer::IncSlideTime(void)
{
  if(ImageSetup.SSsec < 300) {
	    ImageSetup.SSsec++;
	}
}

void cImagePlayer::DecSlideTime(void)
{
  if(ImageSetup.SSsec > 1) {
	    ImageSetup.SSsec--;
	}
}

bool cImagePlayer::ChangeImage(const char *szChange)
{
  if(szChange 
    && m_pCurImage) {
    return Convert(m_pCurImage->Name(), szChange);
  }
  return false;
}

bool cImagePlayer::Convert(const char *szFileName, const char *szChange)
{
  cShellWrapper* pCmd = new cShellWrapper;
  asprintf(&pCmd->szMpg, "%s/%s.mpg", ImageSetup.TempDir,szFileName);
  asprintf(&pCmd->szCmd, "%s \"%s\" \"%s\" %s", g_szConvertScript,
      szFileName, ImageSetup.TempDir, szChange ? szChange : "");
  Exec(pCmd);
  return true;
}

bool cImagePlayer::ConvertZoom(const char *szFileName, const char *szChange, int xRes,
			       int yRes, int xPos, int yPos)
{
  cShellWrapper* pCmd = new cShellWrapper;
  asprintf(&pCmd->szMpg, "%s/%sZ.mpg", ImageSetup.TempDir,szFileName);
  asprintf(&pCmd->szCmd, "%s \"%s\" \"%s\" %s %d %d %d %d",
      g_szZoomScript, szFileName, ImageSetup.TempDir,
      szChange ? szChange : "Original", xRes, yRes, xPos, yPos);
  Exec(pCmd);
  return true;
}

bool cImagePlayer::ConvertJump(int nOffset)
{
  const unsigned int MAX_BILDER = 9;
  static char FileName[MAX_BILDER][PATH_MAX];
  unsigned int i;
  cImage *newcurr;
  int nBilder = 0;

  for (i = 0; i < MAX_BILDER; ++i)
    memset(FileName[i], 0, PATH_MAX);
  
  if(m_pCurImage && m_nCurrentImage > 0) {
  
	  int nJumpFirst = m_nCurrentImage + nOffset;

      if(nJumpFirst > ((int)(m_nTotalImages - MAX_BILDER + 1)))
		  nJumpFirst = ((int)(m_nTotalImages - MAX_BILDER + 1));
	  if(nJumpFirst < 1)
		  nJumpFirst = 1;
	  
	  m_nCurrentImage = nJumpFirst;
      if(m_nCurrentImage > m_nTotalImages)
		  m_nCurrentImage = m_nTotalImages;
	  if(m_nCurrentImage < 1)
		  m_nCurrentImage = 1;

    newcurr = m_pCurSlideShow->cList < cImage >::First();
    for (int n = 1; n < nJumpFirst && newcurr; ++n) 
      newcurr = m_pCurSlideShow->cList < cImage >::Next(newcurr);
	  for (i = 0; (i < MAX_BILDER) && ((nJumpFirst + i) <= m_nTotalImages) && newcurr; ++i,++nBilder) {

      strncpy(FileName[i], newcurr->Name(), sizeof(FileName[0]));
	  dsyslog("image: File%d: %s", i, newcurr->Name());
	  if((nJumpFirst + i) == m_nCurrentImage)
		   m_pCurImage = newcurr;

	  newcurr = m_pCurSlideShow->cList < cImage >::Next(newcurr);
	  }
  }
  

  cShellWrapper* pCmd = new cShellWrapper;
  asprintf(&pCmd->szMpg,"%s/%sZ.mpg", ImageSetup.TempDir,FileName[0]);
  asprintf(&pCmd->szCmd, "%s \"%s\" %s %s \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"",
       g_szJumpScript, ImageSetup.TempDir,
       (nBilder > 6) ? "3x3" : ((nBilder > 3) ? "3x2" : "3x1"),
       "256x192", FileName[0], FileName[1], FileName[2],
       FileName[3], FileName[4], FileName[5], FileName[6],
       FileName[7], FileName[8]);
  Exec(pCmd);
  return true;
}



bool cImagePlayer::LoadFrame(const char* szImageMpg,char** szErr)
{
  bool bRet = false;
  
  int filefd;
  ssize_t nRead;
  
  struct stat st;
  unsigned char* pImage = NULL;
  
  if((filefd = open(szImageMpg, O_RDONLY)) >= 0) {
    dsyslog("file %s opened",szImageMpg);
    // read file state
    if( 0 == fstat(filefd, &st)) {
    
      if(st.st_size > 100) {
        pImage = (unsigned char *)malloc(st.st_size);
  
        if(pImage) {
          nRead = read(filefd, pImage, st.st_size);
          dsyslog("image: %d bytes fread from %s", nRead, szImageMpg);
  
        if(nRead == st.st_size) {
            bRet = SendFrame(pImage, st.st_size);
          }
          else {
            if(nRead  == -1) {
              asprintf(szErr,"Read converted file %s failed. (%d) %s", basename(FileName()) ,errno, strerror(errno));
              esyslog("image: Read converted file %s failed. (%d) %s", szImageMpg ,errno, strerror(errno));
            }
            else {
              asprintf(szErr,"Can't read converted file %s it has wrong size %d, should %d", basename(FileName()), nRead,(unsigned int)st.st_size);
              esyslog("image: Can't read converted file %s it has wrong size %d, should %d", szImageMpg, nRead,(unsigned int)st.st_size);
            }  
          }
    
          free(pImage);
        }
        else {
          asprintf(szErr,"alloc of %d bytes failed",(unsigned int)st.st_size);
          esyslog("image: %s",*szErr);
        }
      } // st.st_size <= 100
      else {
        asprintf(szErr,"Converted file %s it's to small", basename(FileName()));
        esyslog("image: Converted file %s it's to small", szImageMpg);
      }
    } //fstat failed
    else {
      asprintf(szErr,"Read converted file %s failed. (%d) %s", basename(FileName()) ,errno, strerror(errno));
      esyslog("image: Read converted file %s failed. (%d) %s", szImageMpg ,errno, strerror(errno));
    }
    close(filefd);
  }
  else {
    asprintf(szErr,"Read converted file %s failed. (%d) %s", basename(FileName()) ,errno, strerror(errno));
    esyslog("image: Read converted file %s failed. (%d) %s", szImageMpg ,errno, strerror(errno));
  }
  return bRet;
}

bool cImagePlayer::SendFrame(const unsigned char* pData,unsigned int nLength)
{
  static cMutex Mutex;
  cMutexLock lock(&Mutex);

  if(ImageSetup.UseDeviceStillPicture){
    DeviceStillPicture(pData, nLength);
  }
  else  {
    for (int i = 1; i <= 25; ++i)
      SendPESPacket(pData, nLength, i);
  }

  return true;
}

void cImagePlayer::SendPESPacket(const unsigned char *data, int len, int timestamp)
{
  const unsigned int PES_MAX_SIZE=2048;
  unsigned int ptslen = timestamp ? 5 : 1;
  static unsigned char pes_header[PES_MAX_SIZE];

  pes_header[0] = pes_header[1] = 0;
  pes_header[2] = 1;
  pes_header[3] = 0xe0;

  while(len > 0)
  {
    unsigned int payload_size = len;
    if(6 + ptslen + payload_size > PES_MAX_SIZE)
  		payload_size = PES_MAX_SIZE - (6 + ptslen);

    pes_header[4] = (ptslen + payload_size) >> 8;
    pes_header[5] = (ptslen + payload_size) & 255;

    if(ptslen == 5) {
      int x;
      x = (0x02 << 4) | (((timestamp >> 30) & 0x07) << 1) | 1;
      pes_header[8] = x;
      x = ((((timestamp >> 15) & 0x7fff) << 1) | 1);
      pes_header[7] = x >> 8;
      pes_header[8] = x & 255;
      x = ((((timestamp) & 0x7fff) < 1) | 1);
      pes_header[9] = x >> 8;
      pes_header[10] = x & 255;
    } 
    else  {
      pes_header[6] = 0x0f;
    }

    memcpy(&pes_header[6 + ptslen], data, payload_size);
    PlayVideo(pes_header, 6 + ptslen + payload_size);

    len -= payload_size;
    data += payload_size;
    ptslen = 1;
  }
}

////////////////////////////////////////////////////////////////////////////////
/** void cImagePlayer::FaultImage()
Show a precompiled Image if LoadImage can't find a converted Image
@param  - char* szErr
@return - nothing */
void cImagePlayer::FaultImage(const char* szErr)
{
    SendFrame(cErrorImage, sizeof(cErrorImage));
    if(szErr)
      OSD_ErrorMsg(szErr);
}

const char* cImagePlayer::FileName(void) const
{ 
  return m_pCurImage?m_pCurImage->Name():NULL;
}

void cImagePlayer::Exec(cShellWrapper* pCmd)
{
  if(pCmd->szMpg)
    m_bConvertRunning = true;
  if(pCmd) {
    cMutexLock lock(&m_Mutex);
    if(!m_queue.add(pCmd))
      delete pCmd;          //Queue full or Thread is dead     
    m_condVar.Broadcast();  //Wake Thread up
  }
}

void cImagePlayer::Action(void)
{
  bool bGoSleep;
  dsyslog("image: cImagePlayer thread started (pid=%d)", getpid());

  while (m_bActive) {
  
    cShellWrapper *pShell = NULL;
    // Protect the queue ++
    { 
      cMutexLock lock(&m_Mutex);
      if(!m_queue.empty()) {
        pShell = *m_queue.begin();
        m_queue.erase(m_queue.begin());
      }
      bGoSleep = m_queue.empty();
    } // ++
  
    if(pShell) {
    
      if(pShell->szCmd) {
        dsyslog("imageplugin: executing command '%s'", pShell->szCmd);
      
        ImageSetup.SetEnv();
        int nRet = SystemExec(pShell->szCmd);
        if(pShell->szMpg) {
          m_bConvertRunning = false;
        if(0 > nRet) {
          esyslog("imageplugin: can't execute command '%s'", pShell->szCmd);
          if(pShell->szMpg) {
            FaultImage(pShell->szErr);        
            }
        }
        else {
          if(pShell->szMpg
            && (0 != nRet || !LoadFrame(pShell->szMpg,&pShell->szErr)))
              FaultImage(pShell->szErr);
            }
        } 
      }
      delete pShell;
    }
  
    // Go sleep, for one minute
    if(bGoSleep) { 
      cMutexLock lock(&m_Mutex);
      m_condVar.TimedWait(m_Mutex, 60000); 
    }
  }
  
  dsyslog("image: cImagePlayer thread closed (pid=%d)", getpid());
}
