/*
 * Image plugin to VDR (C++)
 *
 * (C) 2004 Andreas Brachold    <vdr04-at-deltab.de>
 *     2003 Kai Tobias Burwieck <kai@burwieck.net>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#ifndef ___DVB_IMAGE_H
#define ___DVB_IMAGE_H

#include <vector>
#define __STL_CONFIG_H

#include <vdr/thread.h>
#include <vdr/player.h>

class cImage;
class cSlideShow;    


struct cShellWrapper {
    
  char* szCmd;
  char* szMpg;
  char* szErr;

  cShellWrapper()
  : szCmd(NULL)
  , szMpg(NULL)
  , szErr(NULL) {
  }

  virtual ~cShellWrapper() {
    if(szErr)
      free(szErr);
    if(szMpg)
      free(szMpg);
    if(szCmd)
      free(szCmd);
  }
};

struct cShellWrapperQueue
 : public std::vector<cShellWrapper*>
{
  virtual ~cShellWrapperQueue()
  {
    iterator i = begin();
    const_iterator e = end();
    for(;e!=i;++i)
        delete(*i);
    clear();
  }

  inline size_t max_size() const 
  {
    return 64;
  }

  inline bool add(cShellWrapper* pCmd) {
    if(size()<max_size())
    {
      if(NULL == pCmd->szMpg) //Pregeneration
        push_back(pCmd);
      else { 
        // Remove all other viewed images from queue
        iterator i = begin();
        while(end()!=i) {
          if((*i)->szMpg) {
            delete(*i);
            erase(i);
            }
          else
            ++i;
          }
        //Place next viewed image at front of the queue
        insert(begin(),pCmd);
      }
      return true;
    }  
    return false;
  }
  
};

class cImagePlayer
: public cPlayer
, private cThread
{
    
  /** Is the Player active */
  volatile bool               m_bActive;
  /** The current active filelist with the slide*/
  cSlideShow                  *m_pCurSlideShow;
  /** Current number of Image in slide */  
  unsigned int                m_nCurrentImage;
  /** Total number of Images in slide */  
  unsigned int                m_nTotalImages;
  /** Reference to current Image of this player  */
  cImage                      *m_pCurImage;
  volatile bool               m_bConvertRunning;

  cMutex                      m_Mutex;
  cCondVar                    m_condVar;
  cShellWrapperQueue          m_queue;

protected:
  void Exec(cShellWrapper* pCmd);

  bool LoadFrame(const char* szImageMpg,char** szErr);
  bool SendFrame(const unsigned char* pData,unsigned int nLength);
  void SendPESPacket(const unsigned char *data, int len, int timestamp);

  virtual void Activate(bool On);
  virtual void Action(void);
public:

  cImagePlayer(cSlideShow *pCurSlideShow);
  virtual ~ cImagePlayer();
  bool IsConvertRunning() const { return m_bConvertRunning; };
  bool NextImage(int Step);
  bool PrevImage(int Step);
  bool GotoImage(unsigned int nNewPictureIndex);
  
  /** Deliver the current number of viewed Image */
  int ImageCurrent(void) const    { return m_nCurrentImage;     }
  /** Deliver the total number of viewed Image */
  int ImageTotal(void) const      { return m_nTotalImages;      }
  /** Deliver the filename from the current number of viewed Image */
  const char* FileName(void) const;
  /** Show Errorimage if operation failed*/
  void FaultImage(const char* szErr);


  void IncSlideTime(void);
  void DecSlideTime(void);
  bool ChangeImage(const char *szChange);
  bool Convert(const char *filename, const char *szChange);
  bool ConvertZoom(const char *filename, const char *szChange, int xRes, int yRes,
                   int xPos, int yPos);
  bool ConvertJump(int Step);
  
  /** Returns the current and total frame index, optionally snapped to the
  nearest I-frame.*/
  virtual bool GetIndex(int &Current, int &Total, bool SnapToIFrame);
};

#endif				//__DVB_IMAGE_H
