#!/bin/bash
###########################################################
# convert_jump.sh
# script to convert the selected image to mpeg2 still
# for vdr image plugin 
# needs : netpbm-progs, mjpegtools [and a vdr or atleast a dvb ff card and the dvb driver tool test_stillimage]
#
# History:
# 2004-06-27    a.brachold - remove ImageMagick stuff
#                            Remove the last script step copy xxx.mpg to image.mpg
# 2004-06-22    a.brachold - cleanup logging 
###########################################################
[ "z$DEBUG" = "zyes" ] && set -xv

# save parameters for logging
CALLED="CALLED '$0 $*'"

INC_DIR=`dirname $0`
[ ! -e ${INC_DIR}/convert_functions.sh ] && echo "Can't find include file convert_functions.sh" && exit 1
. ${INC_DIR}/convert_functions.sh

CONVERT_TEMPDIR="${1:-"/tmp/image"}"
convert_init $CONVERT_TEMPDIR $CALLED

DIR=$(dirname "$4")
FILE=$(basename "$4")
NEWDIR="${CONVERT_TEMPDIR}/${DIR}"
NEWFILE="${NEWDIR}/${FILE}Z.mpg"
JUMPFILE="${NEWDIR}/JUMP.pnm"
logit "FILE: $FILE"
logit "SRCFILE1: $4"
logit "SRCFILE2: $5"
logit "SRCFILE3: $6"
logit "SRCFILE4: $7"
logit "SRCFILE5: $8"
logit "SRCFILE6: $9"
logit "SRCFILE7: ${10}"
logit "SRCFILE8: ${11}"
logit "SRCFILE9: ${12}"
logit "JUMPFILE: $JUMPFILE"

   index=1
   for i in "$4" "$5" "$6" "$7" "$8" "$9" "${10}" "${11}" "${12}" ; do
      if [ -f "$i" ]; then
         getpnmtool "$i"
         $TO_PNM "$i" | pnmscale -xysize 256 192 |\
            pnmcomp -align=center -valign=middle - "$DUMMY_TILE" > "${CONVERT_TEMPDIR}/${index}"
      else
         cp "$DUMMY_TILE" "${CONVERT_TEMPDIR}/${index}"
      fi
   let index++
   done

   cd "$CONVERT_TEMPDIR"
   if [ "$SHOW_NUMBERS" = "ztrue" ] ; then
      pnmindex -size 217 -noquant -across 3 1 2 3 4 5 6 7 8 9 > "$JUMPFILE"
   else
      pnmcat -black -leftright 1 2 3 > row1
      pnmcat -black -leftright 4 5 6 > row2
      pnmcat -black -leftright 7 8 9 > row3
      pnmcat -black -topbottom row1 row2 row3 > "$JUMPFILE"
      rm row1 row2 row3
   fi
   rm 1 2 3 4 5 6 7 8 9

# $JUMPFILE now contains the 3x3 matrix in pnm format.
# TODO: Make sure resolution is correct 


# This is needed for aspect and rotation
getresolution "$JUMPFILE"


[ ! -d "$NEWDIR" ] && mkdir -p -v "$NEWDIR" | tee -a "$CONVERT_LOGFILE"

convert_imagetompeg "$JUMPFILE" "$NEWFILE" "$FLIP"
rm -f "$JUMPFILE"

# NEWFILE contains the 3x3 matrix file in mpeg
if [ ! -f "$NEWFILE" ] ; then 
  logit "$DATE: ERROR Convert $IMAGEFILE to STILL MPEG2 $NEWFILE not successful"; 
  exit 1
fi

logit "$DATE: $NEWFILE created successfull ...";
exit 0
